#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <eglib.h>
#include <malloc.h>

#ifndef NULL
#define NULL 0
#endif

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
char           *fromspec;
char           *tospec;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;


void readfailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, fromspec);
	exit (1);
}

void writefailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, tospec);
	exit (1);
}

void notimplemented (char *buf)
{
	printf ("%s: %s\n", "not implemented", buf);
	exit (1);
}

void EGcrash ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s\n", buf);
	exit (1);
}

void crash (char *buf)
{
	printf ("%s\n", buf);
	exit (1);
}

int
main (int argc, char *argv[])
{
	int   i;
	int   n;
	char *buf;
	void *counts;
	int   windowdim = 0;                   /* numbeer of dimensions of window */
	int   b0, b1, r0, r1;                                   /* size of window */
	double m, c;
	char   type[20], unit[20];
	char *caps = "capabilities";

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	if (argc < 3)
		crash ("Usage: copywindow fromspec tospec [base range [base range]]");
	fromspec = argv[1];
	tospec = argv[2];
	if (argc >= 5)
	{
		windowdim = 1;
		sscanf (argv[3], "%d", &b0);
		sscanf (argv[4], "%d", &r0);
	}
	if (argc >= 7)
	{
		windowdim = 2;
		sscanf (argv[5], "%d", &b1);
		sscanf (argv[6], "%d", &r1);
	}
		
	if (EGinquireSpectrum (fromspec, &dim, bases, ranges, layouts, types))
		readfailed ();
	if (dim != 1 && dim != 2)
		notimplemented ("spectrum neither one or two dimensional");
	if (windowdim < 1)                    /* then use spectrum base and range */
	{
		b0 = bases[0];
		r0 = ranges[0];
	}
	if (windowdim < 2)
	{
		b1 = bases[1];
		r1 = ranges[1];
	}

	if (dim == 1)
	{
		if (EGcreate1dSpectrum (tospec, 0, r0, layouts[0], types[0]))
			writefailed ();
		counts = malloc (r0 * widths[types[0]]);
		if (EGwrite1dSpectrum (tospec, r0 - 1, 1, counts, types[0]))
			writefailed ();

		if (EGread1dSpectrum (fromspec, b0, r0, counts, types[0]))
			readfailed ();
		if (EGwrite1dSpectrum (tospec, 0, r0, counts, types[0]))
			writefailed ();

		free (counts);
	}
	else if (dim == 2)
	{
		if (EGcreate2dSpectrum (tospec,0,r0,0,r1,layouts[0],types[0]))
			writefailed ();
		counts = malloc (r0 * r1 * widths[types[0]]);
		if (EGwrite2dSpectrum (tospec, r0 - 1, 1, r1 - 1, 1, counts, types[0]))
			writefailed ();

		if (EGread2dSpectrum (fromspec, b0, r0, b1, r1, counts, types[0]))
			readfailed ();
		if (EGwrite2dSpectrum (tospec, 0, r0, 0, r1, counts, types[0]))
			writefailed ();

		free (counts);
	}

	buf = (char *) malloc (256);
	for (n = 1; n <= 16; n++) {
		i = EGreadInformation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteInformation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadAnnotation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteAnnotation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadCalibration (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteCalibration (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadEfficiency (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteEfficiency (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}

	if (windowdim > 0)                    /* then use spectrum base and range */
	{
		if (EGreadCalibration (fromspec, 1, buf))
			readfailed ();
		if (*buf != '\0')
		{
			sscanf (buf, "%s %s %d %lf %lf", type, unit, &n, &c, &m);
			c += m * (double) (b0 - bases[0]);
			sprintf (buf, "%s %s %d %f %f", type, unit, n, c, m);

			if (EGwriteCalibration (tospec, 1, buf))
				writefailed ();
		}
	}
	if (windowdim > 1)
	{
		if (EGreadCalibration (fromspec, 2, buf))
			readfailed ();
		if (*buf != '\0')
		{
			sscanf (buf, "%s %s %d %lf %lf", type, unit, &n, &c, &m);
			c += m * (double) (b1 - bases[1]);
			sprintf (buf, "%s %s %d %f %f", type, unit, n, c, m);

			if (EGwriteCalibration (tospec, 2, buf))
				writefailed ();
		}
	}

	free (buf);
	printf ("copied spectrum %s to spectrum %s\n", fromspec, tospec);
	return (0);
}
