
#include <stdlib.h> // for exit()
#include <stdio.h>  // for sscanf
#include <string.h>

#define FILE_SEP_CHR_WIN '\\'
#define FILE_SEP_CHR_UNX '/'

#ifdef WIN32
#include <direct.h>
#include <Windows.h>

#define GETCWD _getcwd
#define FILE_SEP_STR "\\"
#define FILE_SEP_CHR FILE_SEP_CHR_WIN

#else
#include <unistd.h>

#define GETCWD getcwd
#define FILE_SEP_STR "/"
#define FILE_SEP_CHR FILE_SEP_CHR_UNX
#endif

#include "IgorBin.h"
#include "CrossPlatformFileIO.h"
#include "WaveGlue.h"

#include "eglib.h"

#define then

/*
#define int16  short
#define int32  int
#define int8   char
*/

int ReadWaveFile(const char* waveFilePath);

void readfailed (int i, int j, char *fromspec) {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("Read failed at %d %d %s: %s\n", i, j, buf, fromspec);
   exit (1);
   }

void writefailed (char* tospec) {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("Write failed, %s: %s\n", buf, tospec);
   exit (1);
   }


int isAbsPath(char *path) {
    int isAbs = 0;
	if (*(path+1) == ':')
		isAbs = 1;
	else if (*(path+0) == FILE_SEP_CHR_WIN || *(path+0) == FILE_SEP_CHR_UNX)
		isAbs = 1;

	return isAbs;
}

int main(int argc, char *argv[])
{
      char *prog = argv[0], *filePath = NULL, *specPath = NULL;    // program name & Igor wave file
      int n, i, j, err;
      char* errmsg;
      int t, s, w, h, dim;
      char* buffer1, *ptr1;
      int*  buffer2, *ptr2;
      char *fromspec = NULL, *tospec = NULL;
	  char cwd[1024] = "", tmpPath[1024] = "";
      
      if (argc != 2 && argc != 3) {
          printf(" Usage: %s igor_wave_file [original_midas_spectrum]\n", prog);
          printf(" Usage: %s -h\n", prog);
          exit(1);
      }

	  if (!strcmp(argv[1], "-h")) {
          printf (" Usage: %s igor_wave_file [original_midas_spectrum]\n\n", prog);
		  printf (" If only an igor binary wave file name is given, an uncalibrated MIDAS\n");
		  printf (" 2D spectrum will be created from the wave data. The created spectrum\n");
		  printf (" file name and path is the same as the igor file with \".msf\" either\n");
		  printf (" appended or replacing the \".ibw\" extension if there is one.\n\n");
		  printf (" If a MIDAS spectrum file name is also given, calibration and run\n");
		  printf (" information will be taken from that file. The created spectrum file\n");
		  printf (" name and path is the same as the midas file with \".fit\" replacing\n");
		  printf (" \".tile\" in the name or appended (but before any \".msf\" extension)\n");
		  printf (" if the original was not a tiled spectrum.\n");
#ifdef SOLARIS
		  printf("\n N.B. This functionality has not been implemented for Solaris.\n");
#endif
		  exit (0);
	  }

#ifdef SOLARIS
	printf("%s: This functionality has not been implemented for Solaris.\n", prog);
	exit (1);
#endif

	  filePath = *++argv;

	  if (argc == 2) {
		  char *end;
		  strcpy(tmpPath, filePath);
		  end = &tmpPath[0] + strlen(tmpPath) - 4;
		  if (!strcmp(end, ".ibw"))
			  strcpy(end, ".msf");
		  else
			  strcpy(end+4, ".msf");
		  if (isAbsPath(filePath)) {
			  tospec = &tmpPath[0];
		  } else {
			  tospec = GETCWD(cwd, 255);
			  if (cwd[strlen(cwd)-1] != FILE_SEP_CHR_WIN && cwd[strlen(cwd)-1] != FILE_SEP_CHR_UNX)
				  strcat(cwd, FILE_SEP_STR);
			  strcat (cwd, tmpPath);
		  }
	  } else {  // argc == 3
		  char *end;
		  int done = 0;
		  specPath = *++argv;
		  strcpy(tmpPath, specPath);
		  if (isAbsPath(specPath)) {
			  fromspec = strcpy(cwd, tmpPath);
		  } else {
			  fromspec = GETCWD(cwd, 255);
			  if (cwd[strlen(cwd)-1] != FILE_SEP_CHR_WIN && cwd[strlen(cwd)-1] != FILE_SEP_CHR_UNX)
				  strcat(cwd, FILE_SEP_STR);
			  strcat (cwd, tmpPath);
			  strcpy(tmpPath, fromspec);
		  }

		  end = &tmpPath[0] + strlen(tmpPath) - 5;
		  if (!strcmp(end, ".tile")) {
			  strcpy(end, ".fit");
			  done = 1;
		  }
		  end = &tmpPath[0] + strlen(tmpPath) - 9;
		  if (!done && !strcmp(end, ".tile.msf")) {
			  strcpy(end, ".fit.msf");
			  done = 1;
		  }
		  end = &tmpPath[0] + strlen(tmpPath) - 4;
		  if (!done && !strcmp(end, ".msf")) {
			  strcpy(end, ".fit.msf");
			  done = 1;
		  }
		  end = &tmpPath[0] + strlen(tmpPath) - 0;
		  if (!done) {
			  strcpy(end, ".fit");
		  }
		  tospec = &tmpPath[0];
	  }

//	  printf (" filePath: %s\n fromspec: %s\n tospec: %s\n", filePath, fromspec, tospec);

	  err = ReadWaveFile(filePath);
	  if (err) {
		  printf ("Could not read source file.\n");
		  exit (1);
	  }
	  t = getWaveType();
	  s = getBytesPerPoint();
	  w = getWaveWidth();
	  h = getWaveHeight();
	  dim = getWaveDimension();

	  if (s == -1 || s == 0 ||
		  t == -1 || w == -1 ||
		  w == 0 || (dim != 1 && dim != 2) || (dim ==2 && h == 0) ) {
		  printf ("This Igor wave file type is not supported.\n");
		  exit (1);
	  }
      
//    convert wave point ordering for use as a histogram

	  if (dim == 1)
		  h = 1;
	  buffer1 = getWaveDataPtr();
	  buffer2 = malloc(w * h * sizeof(int));
	  if (!buffer2) {
		  printf ("Could not allocate working space for conversion.\n");
		  exit (1);
	  }
	  i = EGreadInformation (tospec, 1, (char *)buffer2);
	  if (!i) {
         printf (" Spectrum %s already exists.\n It will not be overwritten.\n", tospec);
		 exit (1);
	  }

	  ptr1 = buffer1;
	  ptr2 = buffer2;
	  for (i = 0; i < w; i++) {
	    ptr1 = buffer1 + (i * s);
        for (j = 0; j < h; j++) {
            *ptr2 = getWaveData(ptr1);
			ptr2++; ptr1 += (w * s);
        }
      }
      
//    and now create a histogram using the wave data

      if (dim == 1)
		  i = EGcreate1dSpectrum (tospec, 0, w, 0, 5);
	  else
		  i = EGcreate2dSpectrum (tospec, 0, w, 0, h, 0, 5);
      if (i != 0) {
          errmsg = EGerrorMessage (EGerrno);
          printf ("\n Create failed.\n %s: %s\n", errmsg, tospec);
//		  printf (" Does the spectrum already exist? Spectra are not overwritten.\n");
          exit (1);
      }
//      printf("created 2d histogram in %s\n", *argv);

      if (dim == 1)
		  i = EGwrite1dSpectrum (tospec, 0, w, (int*)buffer2, 5);
	  else
		  i = EGwrite2dSpectrum (tospec, 0, w, 0, h, (int*)buffer2, 5);
      if (i != 0) {
          errmsg = EGerrorMessage (EGerrno);
          printf ("\nWrite failed, %s: %s\n", errmsg, tospec);
          exit (1);
      }

	  freeWaveDataPtr();
	  free(buffer2);

	  if (argc == 3) {
		  char *buffer;

		  buffer = (char*)malloc (256);
		  if (!buffer) {
			  printf ("Could not allocate working space for conversion.\n");
			  exit (1);
		  }
		  for (n = 1; n <= 32; n++) {
			  i = EGreadInformation (fromspec, n, buffer);
			  if (i != 0) then readfailed (4,i,fromspec);
			  if (*buffer != '\0')
				  i = EGwriteInformation (tospec, n, buffer);
			  if (i != 0) then writefailed (tospec);
		  }
		  for (n = 1; n <= dim; n++) {
			  i = EGreadAnnotation (fromspec, n, buffer);
			  if (i != 0) then readfailed (5,i,fromspec);
			  if (*buffer != '\0')
				  i = EGwriteAnnotation (tospec, n, buffer);
			  if (i != 0) then writefailed (tospec);
		  }
		  for (n = 1; n <= dim; n++) {
			  i = EGreadCalibration (fromspec, n, buffer);
			  if (i != 0) then readfailed (6,i,fromspec);
			  if (*buffer != '\0')
				  i = EGwriteCalibration (tospec, n, buffer);
			  if (i != 0) then writefailed (tospec);
		  }
		  for (n = 1; n <= dim; n++) {
			  i = EGreadEfficiency (fromspec, n, buffer);
			  if (i != 0) then readfailed (7,i,fromspec);
			  if (*buffer != '\0')
				  i = EGwriteEfficiency (tospec, n, buffer);
			  if (i != 0) then writefailed (tospec);
		  }
		  free (buffer);
	  } else {
		  i = EGwriteInformation(tospec, 1, getWaveName());
		  if (i != 0) then writefailed (tospec);
	  }

//	  printf("Conversion complete\n");
	  printf("%s\n", tospec);
  
  exit(0);
}
