#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <eglib.h>
#include <malloc.h>

#ifndef NULL
#define NULL 0
#endif

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
char           *fromspec;
char           *tospec;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;


void 
readfailed()
{
	char           *buf;
	buf = EGerrorMessage(EGerrno);
	printf("%s: %s\n", buf, fromspec);
	exit(1);
}

void 
writefailed()
{
	char           *buf;
	buf = EGerrorMessage(EGerrno);
	printf("%s: %s\n", buf, tospec);
	exit(1);
}

void 
notimplemented(char *buf)
{
	printf("%s: %s\n", "not implemented", buf);
	exit(1);
}

void 
EGcrash()
{
	char           *buf;
	buf = EGerrorMessage(EGerrno);
	printf("%s\n", buf);
	exit(1);
}

void 
crash(char *buf)
{
	printf("%s\n", buf);
	exit(1);
}

int
main(int argc, char *argv[])
{

/*	//connect to the experiment defined by the EXPERIMENTDIR environment
		// variable
*/

	int   i;
	int   n;
	char *buf;
	int  *counts;
	int   maxcount = 0;
	char *caps = "capabilities";

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	if (argc != 3)
		crash("Usage: copyspectrum fromspec tospec");
	fromspec = argv[1];
	tospec = argv[2];

	i = EGinquireSpectrum(fromspec, &dim, bases, ranges, layouts, types);
	if (i != 0)
	           readfailed();
	if (dim != 1 && dim != 2)
		notimplemented("spectrum neither one or two dimensional");
	if (dim == 1)
	{
		i = EGcreate1dSpectrum(tospec, bases[0], ranges[0], layouts[0], types[0]);
		if (i != 0)
		           writefailed();
		counts = (int *) malloc(ranges[0] * widths[types[0]]);
		i = EGwrite1dSpectrum(tospec, bases[0] + ranges[0] - 1, 1, counts, types[0]);
		if (i != 0)
			writefailed();
		i = EGread1dSpectrum(fromspec, bases[0], ranges[0], counts, types[0]);
		if (i != 0)
			readfailed();

		for (i = 0; i < ranges[0]; i++)            /* find largest count */
			if (counts[i] > maxcount)
				maxcount = counts[i];	
		
		for (i = 0; i < ranges[0]; i++)            /* invert spectrum */
			counts[i] = maxcount - counts[i];

		i = EGwrite1dSpectrum(tospec, bases[0], ranges[0], counts, types[0]);
		if (i != 0)
			writefailed();
		free((char *) counts);
		}

	if (dim == 2)
	{
		i = EGcreate2dSpectrum(tospec, bases[0], ranges[0], bases[1], ranges[1],
				       layouts[0], types[0]);
		if (i != 0)
			writefailed();
		counts = (int *) malloc(ranges[0] * ranges[1] * widths[types[0]]);
		i = EGwrite2dSpectrum(tospec, bases[0] + ranges[0] - 1, 1,
				      bases[1] + ranges[1] - 1, 1,
				      counts, types[0]);
		if (i != 0)
			writefailed();
		i = EGread2dSpectrum(fromspec, bases[0], ranges[0], bases[1], ranges[1], counts, types[0]);
		if (i != 0)
			readfailed();

		for (i = 0; i < ranges[0] * ranges[1]; i++)            /* find largest count */
			if (counts[i] > maxcount)
				maxcount = counts[i];	
		
		for (i = 0; i < ranges[0] * ranges[1]; i++)            /* invert spectrum */
			counts[i] = maxcount - counts[i];

		i = EGwrite2dSpectrum(tospec, bases[0], ranges[0], bases[1], ranges[1], counts, types[0]);
		if (i != 0)
			writefailed();
		free((char *) counts);
		}

	buf = (char *) malloc(256);
	for (n = 1; n <= 16; n++) {
		i = EGreadInformation(fromspec, n, buf);
		if (i != 0)
			readfailed();
		if (*buf != '\0')
			i = EGwriteInformation(tospec, n, buf);
		if (i != 0)
			writefailed();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadAnnotation(fromspec, n, buf);
		if (i != 0)
			readfailed();
		if (*buf != '\0')
			i = EGwriteAnnotation(tospec, n, buf);
		if (i != 0)
			writefailed();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadCalibration(fromspec, n, buf);
		if (i != 0)
			readfailed();
		if (*buf != '\0')
			i = EGwriteCalibration(tospec, n, buf);
		if (i != 0)
			writefailed();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadEfficiency(fromspec, n, buf);
		if (i != 0)
			readfailed();
		if (*buf != '\0')
			i = EGwriteEfficiency(tospec, n, buf);
		if (i != 0)
			writefailed();
	}
	free(buf);
	printf("copied spectrum %s to spectrum %s\n", fromspec, tospec);
	return 0;
}
