#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>

#include "eglib.h"
#include "nsflib.h"

#define FUNCTION
#define then
#define doo
#ifndef NULL
#define NULL 0
#endif

int widths[] = {1, 1, 2, 2, 4, 4, 4};
char* dir;
char* fromspec;
char* tospec;
int bases[8], new_bases[4];
int ranges[8], new_ranges[4];
int layouts[2];
int types[2];
int dim;


void readfailed () {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("Read failed %s: %s\n", buf, fromspec);
//   exit (1);
   }

void writefailed () {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("Write failed, %s: %s\n", buf, tospec);
   exit (1);
   }

void notimplemented (char* buf) {
   printf ("%s: %s\n", "not implemented", buf);
   exit (1);
   }

void EGcrash () {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("%s\n", buf);
   exit (1);
   }

void crash (char* buf) {
   printf ("%s\n", buf);
   exit (1);
   }

#define MAXCALTERMS 14

void recalibrate (int cp, char* calbuf, char* newcalbuf) {
   int i;
   int len, oldlen;
//   int ierr;
   char type[16];
   char units[16];
   int noterms;
   float terms[MAXCALTERMS];
//   float newterms[MAXCALTERMS];
   double newc, c, m;

   for (i=0; i<16; i++) type[i] = units[i] = '\0';
   *newcalbuf='\0';
   oldlen = len=strlen (calbuf);
   if (len==0) then return;
   for (i=0; i<MAXCALTERMS; i++) terms[i]=0.0;

/*
   sdcal_ (calbuf, &len, type, units, &noterms, terms, &ierr);

   if (noterms<4) then return;
   for (i=0; i<MAXCALTERMS; i++) newterms[i]=terms[i];
   newc=(double)terms[0]+cp*(terms[1]+cp*(terms[2]+cp*terms[3]));
newterms[0] = newc;
printf("newc %g, newterms[0] %g\n", newc, newterms[0]);
   len=255;
   secal_ (newcalbuf, &len, type, units, &noterms, newterms, &ierr, len, strlen(type), strlen(units));
*/
   sscanf (calbuf, "%s %s %d %lf %lf", type, units, &noterms, &c, &m);
   newc=c + m*cp;
   *(newcalbuf+len)='\0';
   sprintf(newcalbuf, "%s %s %d %.4lf %.12lf", type, units, noterms, newc, m);
   }

int main (int argc, char **argv) {

// connect to the experiment defined by the EXPERIMENTDIR environment
// variable

   int i, rs;
   int n, nb=0, nr=0;
   char *buf, *newbuf, *ptr;
   int* counts;
   char *caps = "capabilities";

   dir = (char *)getenv ("EXPERIMENTDIR");
   if (dir==NULL || *dir=='\0') then dir = caps = "";
   i = EGsetExperimentPath (dir);
   if (i != 0) then EGcrash();
   i = EGconnectExperiment (caps);
   if (i != 0) then EGcrash ();

   if (argc < 3 || argc > 9) then crash ("usage: meis-copy-spectrum fromspec tospec [-b base1 [base2]] [-r range1 [range2]]");
   fromspec = *(argv+1);
   tospec = *(argv+2);
   if (!strcmp(fromspec, tospec)) then crash ("don't copy a spectrum onto itself");

   for (i=0; i<4; i++) 
      new_bases[i] = new_ranges[i] = -1;

	argc-=2; argv+=2;
	for (argv++ ; --argc > 0; argv++) {
		if (**argv == '-')  {
			switch ((*argv)[1]) {

			case 'b':  {
				nb = 0;
				while (argc > 1 && *(*(argv+1)) != '-') {
					ptr = *(argv+1);
					rs = sscanf(ptr, "%i", &new_bases[nb]);
					if (rs == 0 || rs == EOF)  {
						new_bases[nb] = -1;
					}
					argc--;argv++;
					if (nb<3) nb++;
				}
				break;
			}

			case 'r':  {
				nr = 0;
				while (argc > 1 && *(*(argv+1)) != '-') {
					ptr = *(argv+1);
					rs = sscanf(ptr, "%i", &new_ranges[nr]);
					if (rs == 0 || rs == EOF)  {
						new_ranges[nr] = -1;
					}
					argc--;argv++;
					if (nr<3) nr++;
				}
				break;
			}

			default:
				break;
			}
		}
	}

   if (nb==3 || nr==3) then crash ("usage: meis-copy-spectrum fromspec tospec [-b base1 [base2]] [-r range1 [range2]]");

   i = EGinquireSpectrum (fromspec, &dim, bases, ranges, layouts, types);
   if (i != 0) then readfailed ();
   if (dim != 1 && dim != 2) then {
      printf("experiment path:%s  %s\n",dir,caps);
      printf("%s dimension=%d base=%d range=%d layout=%d type=%d\n",fromspec, dim, bases[0], ranges[0], layouts[0], types[0]);
      notimplemented ("spectrum neither one nor two dimensional");
   }

   for (i=0; i<dim; i++) {
      if (new_bases[i] == -1)
         new_bases[i] = bases[i];
      if (new_ranges[i] == -1)
         new_ranges[i] = ranges[i];

      if (new_bases[i] < bases[i])
         break;
      if (new_bases[i] >= bases[i]+ranges[i])
         break;
      if (new_ranges[i] > ranges[i])
         break;
      if (new_bases[i] + new_ranges[i] > bases[i] + ranges[i])
         new_ranges[i] = bases[i] + ranges[i] - new_bases[i];

      bases[i] = new_bases[i];
      ranges[i] = new_ranges[i];
   }
   if (i<dim) then crash ("usage: meis-copy-spectrum fromspec tospec [-b base1 [base2]] [-r range1 [range2]]");


   if (dim == 1) then {
      i = EGcreate1dSpectrum (tospec, nb ? 0 : bases[0], ranges[0], layouts[0], types[0]);
      if (i != 0) then writefailed ();
      counts = (int*)malloc (ranges[0] * widths[types[0]]);
      i = EGwrite1dSpectrum (tospec, (nb ? 0 : bases[0]) + ranges[0]-1, 1, counts, types[0]);
      if (i != 0) then writefailed ();
      i = EGread1dSpectrum (fromspec, bases[0], ranges[0], counts, types[0]);
      if (i != 0) then readfailed ();
      i = EGwrite1dSpectrum (tospec, nb ? 0 : bases[0], ranges[0], counts, types[0]);
      if (i != 0) then writefailed ();
      free ((char*)counts);
   }

   if (dim == 2) then {
      i = EGcreate2dSpectrum (tospec, nb ? 0 : bases[0], ranges[0], nb==2 ? 0 : bases[1], ranges[1],
          layouts[0], types[0]);
      if (i != 0) then writefailed ();
      counts = (int*)malloc (ranges[0] * ranges[1] * widths[types[0]]);
      i = EGwrite2dSpectrum (tospec, (nb ? 0 : bases[0]) + ranges[0]-1, 1,
                                    (nb==2 ? 0 : bases[1]) + ranges[1]-1, 1,
                                    counts, types[0]);
      if (i != 0) then writefailed ();
      i = EGread2dSpectrum (fromspec, bases[0], ranges[0], bases[1], ranges[1], counts, types[0]);
      if (i != 0) then readfailed ();
      i = EGwrite2dSpectrum (tospec, nb ? 0 : bases[0], ranges[0], (nb==2 ? 0 : bases[1]), ranges[1], counts, types[0]);
      if (i != 0) then writefailed ();
      free ((char*)counts);
   }

   buf = (char*)malloc (256);
   newbuf = (char*)malloc (256);
   for (n = 1; n <= 32; n++) {
      i = EGreadInformation (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then
         i = EGwriteInformation (tospec, n, buf);
      if (i != 0) then writefailed ();
   }
   for (n = 1; n <= dim; n++) {
      i = EGreadAnnotation (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then
         i = EGwriteAnnotation (tospec, n, buf);
      if (i != 0) then writefailed ();
   }
   for (n = 1; n <= dim; n++) {
      i = EGreadCalibration (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then {
         char *bf = buf;
         if (nb && n<=nb) {
            bf = newbuf;
            recalibrate (bases[n-1], buf, bf);
         }
         i = EGwriteCalibration (tospec, n, bf);
      }
      if (i != 0) then writefailed ();
   }
   for (n = 1; n <= dim; n++) {
      i = EGreadEfficiency (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then
         i = EGwriteEfficiency (tospec, n, buf);
      if (i != 0) then writefailed ();
   }
   free (buf);
   free (newbuf);
   printf ("copied spectrum %s to spectrum %s\n", fromspec, tospec);
   return (0);
}
