/*******************************************************************************
* mergespectra.c
*
* This utility merges a number of 1 or 2 dimensional spectra
* temporary hack to do own compression until spectrum server works
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <float.h>

#ifndef WIN32
#include <macros.h>
#endif

#include "eglib.h"

#define MAX_SPEC 30
#define UNINITIALISED -1

typedef struct spectrum_information {
	char *name;
	int   bases[8];
	int   ranges[8];
	int   layouts[2];
	int   types[2];
	int   dim;
	double m[8];
	double c[8];
} SP_INFO;

/* extern int      EGerrno; */

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
SP_INFO         *fromspec = NULL;
SP_INFO         tospec = {NULL,
                          {0, 0, 0, 0, 0, 0, 0, 0},
                          {0, 0, 0, 0, 0, 0, 0, 0},
                          {0, 0},
                          {0, 0},
                           0,
                          {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
                          {DBL_MAX, DBL_MAX, DBL_MAX, DBL_MAX,
                           DBL_MAX, DBL_MAX, DBL_MAX, DBL_MAX}     };
int             dim = UNINITIALISED;
int             numspecs;
double          maximum[8] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
/*double          minimum[2] = {DBL_MAX, DBL_MAX};*/
int compress[8] = {UNINITIALISED, UNINITIALISED, UNINITIALISED, UNINITIALISED, 
                   UNINITIALISED, UNINITIALISED, UNINITIALISED, UNINITIALISED}; 

void hackCompression (int *from, int *to, int rangeFrom, int rangeTo);

void myexit (int n)
{
	if (fromspec) free (fromspec);
	exit (n);
}

void readfailed (char *from)
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, from);
	myexit (1);
}

void writefailed (char *to)
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, to);
	myexit (1);
}

void notimplemented (char *buf)
{
	printf ("%s: %s\n", "not implemented", buf);
	myexit (1);
}

void EGcrash ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s\n", buf);
	myexit (1);
}

void crash (char *buf)
{
	printf ("%s\n", buf);
	myexit (1);
}

int
main (int argc, char *argv[])
{
	int   i;
	int   n;
	int debug = 0;
	char  buf[256];
	void *counts;
#ifndef SAS_COMPRESSION
	void *compCounts;
#endif
	int   windowdim = 0;                   /* number of dimensions of window */
	char   type[20], unit[8][20];
	SP_INFO *s;
	char *caps = "capabilities";
	char *argv0 = argv[0];

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		 EGcrash ();

if (argc >= 2 && !strcmp(argv[1], "-d")) {debug = 1; argc--; argv++;}

	if (argc < 3) {
		sprintf (buf, "Usage: %s fromspec [fromspec ...] tospec", argv0);
		crash (buf);
	}

	numspecs = argc-2;
/* allocate space for 1 more than necessary so we end with a valid null pointer */
	fromspec = calloc(numspecs+1, sizeof (SP_INFO));
	tospec.name = argv[--argc];
	for (s = fromspec; --argc; s++)     /* find sizes of spectra to be merged */
	{
		s->name = *++argv;
		if (EGinquireSpectrum (s->name, &s->dim, s->bases, s->ranges,
		                                s->layouts, s->types))
			readfailed (s->name);

		if (dim == UNINITIALISED)                          /* check dimension */
			dim = s->dim;
		else if (dim != s->dim)
			crash ("spectra have different dimensions");
		else if (dim > 2)
			notimplemented ("spectrum neither one or two dimensional");

		for (i = 0; i < dim; i++)                  /* get calibration strings */
		{
			if (EGreadCalibration (s->name, i + 1, buf))
				readfailed (s->name);
			if (*buf != '\0')
			{
				sscanf (buf, "%s %s %d %lf %lf", type, unit[i], &n, &s->c[i], &s->m[i]);
				tospec.m[i] = max (tospec.m[i], s->m[i]);
				tospec.c[i] = min (tospec.c[i], s->c[i] + s->m[i] * s->bases[i]);
				maximum[i] = max (maximum[i], s->c[i] + s->m[i] * (s->bases[i] + s->ranges[i]));
			}
		}
	}

	tospec.bases[0] = 0;
	tospec.ranges[0] = (int) ((maximum[0] - tospec.c[0]) / tospec.m[0] + 0.5);
	tospec.layouts[0] = fromspec->layouts[0];
	tospec.types[0] = fromspec->types[0];

	if (dim == 1)
	{
		if (EGcreate1dSpectrum (tospec.name, 0, tospec.ranges[0],
		                        tospec.layouts[0], tospec.types[0]))
			writefailed (tospec.name);

		counts = malloc (tospec.ranges[0] * widths[tospec.types[0]]);
		if (EGwrite1dSpectrum (tospec.name, tospec.ranges[0] - 1, 1,
		                       counts, tospec.types[0]))
			writefailed (tospec.name);

		for (s = fromspec; s->name; s++)
		{
			if (EGread1dSpectrum (s->name, s->bases[0], s->ranges[0],
			                      counts, s->types[0]))
				readfailed (s->name);
			if (EGreadCalibration (s->name, 1, buf))
				readfailed (s->name);
			if (*buf != '\0')
			{
				sscanf (buf, "%s %s %d %lf %lf", type, unit[0], &n, &s->c[0], &s->m[0]);
				tospec.bases[0] = (int) ((s->c[0] - tospec.c[0]) / s->m[0] + 0.5);
			}

			if (EGwrite1dSpectrum (tospec.name, tospec.bases[0], s->ranges[0],
			                       counts, tospec.types[0]))
				writefailed (tospec.name);
		}
	}
	else if (dim == 2)
	{
		tospec.bases[1] = 0;
		tospec.ranges[1] = (int) ((maximum[1] - tospec.c[1]) /tospec.m[1] + 0.5);

		if (EGcreate2dSpectrum (tospec.name, 0, tospec.ranges[0],
		                                     0, tospec.ranges[1],
		                        tospec.layouts[0], tospec.types[0]))
			writefailed (tospec.name);

		counts = malloc (tospec.ranges[0] * tospec.ranges[1] * 
		                 widths[tospec.types[0]]);
#ifndef SAS_COMPRESSION
		compCounts = malloc (tospec.ranges[0] * tospec.ranges[1] * 
		                     widths[tospec.types[0]]);
#endif
		if (EGwrite2dSpectrum (tospec.name, tospec.ranges[0] - 1, 1,
		                                    tospec.ranges[1] - 1, 1,
		                                    counts, tospec.types[0]))
			writefailed (tospec.name);

		for (s = fromspec; s->name; s++)
		{
			for (i = 0; i < dim; i++)
			{
				tospec.bases[i] = (int) ((s->c[i] - tospec.c[i]) / tospec.m[i] + 0.5);
				compress[i] = (int) ((double) s->ranges[i] * s->m[i] / tospec.m[i] + 0.5);
			}
#ifdef SAS_COMPRESSION
			EGsetDefaultCompression (compress);

			if (EGread2dSpectrum (s->name, s->bases[0], s->ranges[0],
			                               s->bases[1], s->ranges[1],
			                               counts, s->types[0]))
				readfailed (s->name);

			if (EGwrite2dSpectrum (tospec.name, tospec.bases[0], compress[0],
			                                    tospec.bases[1], compress[1],
			                                    counts, tospec.types[0]))
				writefailed (tospec.name);
#else
			for (i = 0; i < s->ranges[0]; i++)/* do it column by column */
			{
				if (EGread2dSpectrum (s->name, s->bases[0] + i, 1,
			                                   s->bases[1], s->ranges[1],
				                               counts, s->types[0]))
					readfailed (s->name);

				hackCompression (counts, compCounts, s->ranges[1], compress[1]);

				if (EGwrite2dSpectrum (tospec.name,
				                       tospec.bases[0] + i, 1,
				                       tospec.bases[1], compress[1],
				                       compCounts, tospec.types[0]))
					writefailed (tospec.name);
			}

#endif
		}
	}

	for (n = 1; n <= 16; n++)
	{
		if (EGreadInformation (fromspec->name, n, buf))
			readfailed (fromspec->name);
		if (*buf != '\0')
			if (EGwriteInformation (tospec.name, n, buf))
				writefailed (tospec.name);
	}
	for (n = 1; n <= dim; n++)
	{
		if (EGreadAnnotation (fromspec->name, n, buf))
			readfailed (fromspec->name);
		if (*buf != '\0')
			if (EGwriteAnnotation (tospec.name, n, buf))
				writefailed (tospec.name);
	}
	for (n = 1; n <= dim; n++)
	{
		if (EGreadEfficiency (fromspec->name, n, buf))
			readfailed (fromspec->name);
		if (*buf != '\0')
			if (EGwriteEfficiency (tospec.name, n, buf))
				writefailed (tospec.name);
	}
	for (i = 0; i < dim; i++)
	{
		sprintf (buf,"%s %s 2 %f %f", type, unit[i], tospec.c[i], tospec.m[i]);
		if (EGwriteCalibration (tospec.name, i + 1, buf))
			writefailed (tospec.name);
	}

	printf ("merged %d spectra into spectrum %s\n", numspecs, tospec.name);
	myexit (0);
}
#ifndef SAS_COMPRESSION

int proportion (int count, double overlap)
{
	return (int) ((double) count * overlap + 0.5);
}

void hackCompression (int *from, int *to, int rangeFrom, int rangeTo)
{
	int t, f;
	double minTo, maxTo;
	double minFrom, maxFrom;
	double hi, lo;

	memset (to, 0, rangeTo * 4);                         /* initialise memory */

	for (t = f = 0; t < rangeTo; t++)
	{
		minTo = (double)  t      / (double) rangeTo;
		maxTo = (double) (t + 1) / (double) rangeTo;
		if (f)
			f--;

		for (; f < rangeFrom; f++)          /* extract counts from raw buffer */
		{
			minFrom = (double)  f      / (double) rangeFrom;
			maxFrom = (double) (f + 1) / (double) rangeFrom;

			if (minFrom >= maxTo)  /* then bin complete - move on to next one */
				break;
			else if (maxFrom <= minTo)  /* then no overlap - fetch next count */
				continue;

			hi = min (maxTo, maxFrom);
			lo = max (minTo, minFrom);
			to[t] += proportion (from[f], (hi - lo) / (maxFrom - minFrom));
		}
	}
}
#endif
