#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>

#include "eglib.h"
/* #include "nsflib.h" */

#define FUNCTION
#define then
#define doo
#ifndef NULL
#define NULL 0
#endif

int widths[] = {sizeof(unsigned char),
				sizeof(char),
				sizeof(unsigned short),
				sizeof(short),
				sizeof(unsigned long),
				sizeof(long),
				sizeof(float)
				};

char* dir;
char* fromspec;
char* tospec;
int bases[8];
int ranges[8];
int layouts[2];
int types[2];
int dim;


void readfailed () {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("Read failed %s: %s\n", buf, fromspec);
//   exit (1);
   }

void writefailed () {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("Write failed, %s: %s\n", buf, tospec);
   exit (1);
   }

void notimplemented (char* buf) {
   printf ("%s: %s\n", "not implemented", buf);
   exit (1);
   }

void EGcrash () {
   char* buf;
   buf = EGerrorMessage (EGerrno);
   printf ("%s\n", buf);
   exit (1);
   }

void crash (char* buf) {
   printf ("%s\n", buf);
   exit (1);
   }

int main (int argc, char **argv) {

// connect to the experiment defined by the EXPERIMENTDIR environment
// variable

   float factor = 1.0, threshold = 0.0;
   int i, j, rs;
   int xthreshold=0, doThresh = 0;
   int n, nb=0, nr=0;
   char *buf, *ptr;
   int* counts;
   char *caps = "capabilities";

   dir = (char *)getenv ("EXPERIMENTDIR");
   if (dir==NULL || *dir=='\0') then dir = caps = "";
   i = EGsetExperimentPath (dir);
   if (i != 0) then EGcrash();
   i = EGconnectExperiment (caps);
   if (i != 0) then EGcrash ();

   if (argc < 3 || argc > 7) then crash ("usage: multiply-spectrum fromspec tospec [-f factor] [-t threshold]");
   fromspec = *(argv+1);
   tospec = *(argv+2);
   if (!strcmp(fromspec, tospec)) then crash ("don't copy a spectrum onto itself");

	argc-=2; argv+=2;
	for (argv++ ; --argc > 0; argv++) {
		if (**argv == '-')  {
			switch ((*argv)[1]) {

			case 'f':  {
				if (argc > 1 && *(*(argv+1)) != '-') {
					ptr = *(argv+1);
					rs = sscanf(ptr, "%f", &factor);
					if (rs == 0 || rs == EOF)  {
						factor = 1.0;
					}
					argc--;argv++;
				} else {
					nb = 3;
				}
				break;
			}

			case 't':  {
				if (argc > 1 && *(*(argv+1)) != '-') {
					doThresh = 1;
					ptr = *(argv+1);
					rs = sscanf(ptr, "%f", &threshold);
					if (rs == 0 || rs == EOF)  {
						doThresh = 0;
					}
					argc--;argv++;
				} else {
					nr = 3;
				}
				break;
			}

			default:
				break;
			}
		}
	}

   if (nb==3 || nr==3) then crash ("usage: multiply-spectrum fromspec tospec [-f factor] [-t threshold]");

   i = EGinquireSpectrum (fromspec, &dim, bases, ranges, layouts, types);
   if (i != 0) then readfailed ();
   if (dim != 1 && dim != 2) then {
      printf("experiment path:%s  %s\n",dir,caps);
      printf("%s dimension=%d base=%d range=%d layout=%d type=%d\n",fromspec, dim, bases[0], ranges[0], layouts[0], types[0]);
      notimplemented ("spectrum neither one nor two dimensional");
   }

   if (dim == 1) then {
      i = EGcreate1dSpectrum (tospec, nb ? 0 : bases[0], ranges[0], layouts[0], types[0]);
      if (i != 0) then writefailed ();
      counts = (int*)malloc (ranges[0] * widths[types[0]]);
      i = EGwrite1dSpectrum (tospec, (bases[0]) + ranges[0]-1, 1, counts, types[0]);
      if (i != 0) then writefailed ();
      i = EGread1dSpectrum (fromspec, bases[0], ranges[0], counts, types[0]);
      if (i != 0) then readfailed ();
	  switch (types[0]) {
	      case 0:		// unsigned 8 bit
		  {
			  unsigned char *c = (unsigned char *)counts;
			  for (j=0; j<ranges[0]; j++) {
				  if (*c>threshold)
					  *c = (unsigned char) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	  	  case 1:		// signed 8 bit
		  {
			  char *c = (char *)counts;
			  for (j=0; j<ranges[0]; j++) {
				  if (*c>threshold)
					  *c = (char) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	      case 2:		// unsigned 16 bit
		  {
			  unsigned short *c = (unsigned short *)counts;
			  for (j=0; j<ranges[0]; j++) {
				  if (*c>threshold)
					  *c = (unsigned short) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	  	  case 3:		// signed 16 bit
		  {
			  short *c = (short *)counts;
			  for (j=0; j<ranges[0]; j++) {
				  if (*c>threshold)
					  *c = (short) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	      case 4:		// unsigned 32 bit
		  {
			  unsigned long *c = (unsigned long *)counts;
			  for (j=0; j<ranges[0]; j++) {
				  if (*c>threshold)
					  *c = (unsigned long) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	  	  case 5:		// signed 32 bit
		  {
			  long *c = (long *)counts;
			  for (j=0; j<ranges[0]; j++) {
				  if (*c>threshold)
					  *c = (long) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	      case 6:		// float
		  {
			  float *c = (float *)counts;
			  for (j=0; j<ranges[0]; j++) {
				  if (*c>threshold)
					  *c = (float) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

		  default:
		  break;
	  }
      i = EGwrite1dSpectrum (tospec, bases[0], ranges[0], counts, types[0]);
      if (i != 0) then writefailed ();
      free ((char*)counts);
   }

   if (dim == 2) then {
      i = EGcreate2dSpectrum (tospec, bases[0], ranges[0], bases[1], ranges[1],
          layouts[0], types[0]);
      if (i != 0) then writefailed ();
      counts = (int*)malloc (ranges[0] * ranges[1] * widths[types[0]]);
      i = EGwrite2dSpectrum (tospec, bases[0] + ranges[0]-1, 1,
                                     bases[1] + ranges[1]-1, 1,
                                    counts, types[0]);
      if (i != 0) then writefailed ();
      i = EGread2dSpectrum (fromspec, bases[0], ranges[0], bases[1], ranges[1], counts, types[0]);
      if (i != 0) then readfailed ();
	  switch (types[0]) {
	      case 0:		// unsigned 8 bit
		  {
			  unsigned char *c = (unsigned char *)counts;
			  for (j=0; j<ranges[0] * ranges[1]; j++) {
				  if (*c>threshold)
					  *c = (unsigned char) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	  	  case 1:		// signed 8 bit
		  {
			  char *c = (char *)counts;
			  for (j=0; j<ranges[0] * ranges[1]; j++) {
				  if (*c>threshold)
					  *c = (char) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	      case 2:		// unsigned 16 bit
		  {
			  unsigned short *c = (unsigned short *)counts;
			  for (j=0; j<ranges[0] * ranges[1]; j++) {
				  if (*c>threshold)
					  *c = (unsigned short) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	  	  case 3:		// signed 16 bit
		  {
			  short *c = (short *)counts;
			  for (j=0; j<ranges[0] * ranges[1]; j++) {
				  if (*c>threshold)
					  *c = (short) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	      case 4:		// unsigned 32 bit
		  {
			  unsigned long *c = (unsigned long *)counts;
			  for (j=0; j<ranges[0] * ranges[1]; j++) {
				  if (*c>threshold)
					  *c = (unsigned long) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	  	  case 5:		// signed 32 bit
		  {
			  long *c = (long *)counts;
			  for (j=0; j<ranges[0] * ranges[1]; j++) {
				  if (*c>threshold)
					  *c = (long) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

	      case 6:		// float
		  {
			  float *c = (float *)counts;
			  for (j=0; j<ranges[0] * ranges[1]; j++) {
				  if (*c>threshold)
					  *c = (float) ((double) *c * factor + (double)0.5);
				  c++;
			  }
		  }
		  break;

		  default:
		  break;
	  }
      i = EGwrite2dSpectrum (tospec, bases[0], ranges[0], bases[1], ranges[1], counts, types[0]);
      if (i != 0) then writefailed ();
      free ((char*)counts);
   }

   buf = (char*)malloc (256);
   for (n = 1; n <= 32; n++) {
      i = EGreadInformation (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then
         i = EGwriteInformation (tospec, n, buf);
      if (i != 0) then writefailed ();
   }
   for (n = 1; n <= dim; n++) {
      i = EGreadAnnotation (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then
         i = EGwriteAnnotation (tospec, n, buf);
      if (i != 0) then writefailed ();
   }
   for (n = 1; n <= dim; n++) {
      i = EGreadCalibration (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then
         i = EGwriteCalibration (tospec, n, buf);
      if (i != 0) then writefailed ();
   }
   for (n = 1; n <= dim; n++) {
      i = EGreadEfficiency (fromspec, n, buf);
      if (i != 0) then readfailed ();
      if (*buf != '\0') then
         i = EGwriteEfficiency (tospec, n, buf);
      if (i != 0) then writefailed ();
   }
   free (buf);
   if (doThresh)
	   printf ("copied spectrum %s to spectrum %s, counts above %g multiplied by %g\n", fromspec, tospec, threshold, factor);
   else
	   printf ("copied spectrum %s to spectrum %s, counts multiplied by %g\n", fromspec, tospec, factor);

   return (0);
}
