#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>

#include "eglib.h"

#ifndef NULL
#define NULL 0
#endif

#define MIN(a,b) (((a)<(b)) ? (a) :(b))
#define MAX(a,b) (((a)>(b)) ? (a) :(b))

#define USAGE1 " (1D): %s -spectrum specname [-format [y|xy|5y]]  [-limit lim1 [lim2]]\n"
#define USAGE2 " (2D): %s -spectrum specname [-format [c|r]]  [-limit lim1 [lim2] [lim3] [lim4]]\n"

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
char           *prog;
char           *fromspec;
char           *tospec;
char           *format;
char           *arglim1;
char           *arglim2;
char           *arglim3;
char           *arglim4;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;


void 
readfailed()
{
	char           *buf;
	buf = EGerrorMessage(EGerrno);
	fprintf(stderr, "%s: %s\n", buf, fromspec);
	exit(1);
}

void 
writefailed()
{
	fprintf(stderr, "Failed to write to output file\n");
	exit(1);
}

void 
notimplemented(char *buf)
{
	fprintf(stderr, "%s: %s\n", "not implemented", buf);
	exit(1);
}

void 
EGcrash()
{
	char           *buf;
	buf = EGerrorMessage(EGerrno);
	fprintf(stderr, "%s\n", buf);
	exit(1);
}

void 
crash(char *buf)
{
	fprintf(stderr, "%s\n", buf);
	exit(1);
}

void 
usage(char *buf)
{
	fprintf(stderr, "%s\n", buf);
	fprintf(stderr, "Usage\t" USAGE1, prog);
	fprintf(stderr, "\t" USAGE2, prog);
	exit(1);
}


void 
usage1(char *buf)
{
	fprintf(stderr, "%s\n", buf);
	fprintf(stderr, "Usage\t" USAGE1, prog);
	exit(1);
}


void 
usage2(char *buf)
{
	fprintf(stderr, "%s\n", buf);
	fprintf(stderr, "Usage\t" USAGE2, prog);
	exit(1);
}




void
printitx (FILE *file, void *buf, int len, int type) {
	int i;

	switch (type) {
		case 0:
			{
			unsigned char *ucbuf = (unsigned char *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12u\n", ucbuf[i]);
			break;
			}
		case 1:
			{
			char *cbuf = (char *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d\n", cbuf[i]);
			break;
			}
		case 2:
			{
			unsigned short *usbuf = (unsigned short *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12u\n", usbuf[i]);
			break;
			}
		case 3:
			{
			short *sbuf = (short *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d\n", sbuf[i]);
			break;
			}
		case 4:
			{
			unsigned long *ulbuf = (unsigned long *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12u\n", ulbuf[i]);
			break;
			}
		case 5:
			{
			long *lbuf = (long *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d\n", lbuf[i]);
			break;
			}
		case 6:
			{
			double *fbuf = (double *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12g\n", fbuf[i]);
			break;
			}
	}
}

void
printitxy (FILE *file, void *buf, int len, int type) {
	int i;

	switch (type) {
		case 0:
			{
			unsigned char *ucbuf = (unsigned char *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d%12u\n", i, ucbuf[i]);
			break;
			}
		case 1:
			{
			char *cbuf = (char *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d%12d\n", i, cbuf[i]);
			break;
			}
		case 2:
			{
			unsigned short *usbuf = (unsigned short *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d%12u\n", i, usbuf[i]);
			break;
			}
		case 3:
			{
			short *sbuf = (short *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d%12d\n", i, sbuf[i]);
			break;
			}
		case 4:
			{
			unsigned long *ulbuf = (unsigned long *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d%12u\n", i, ulbuf[i]);
			break;
			}
		case 5:
			{
			long *lbuf = (long *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d%12d\n", i, lbuf[i]);
			break;
			}
		case 6:
			{
			double *fbuf = (double *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d%12g\n", i, fbuf[i]);
			break;
			}
	}
}

void
printit5y (FILE *file, void *buf, int len, int type) {
	int i, j;

	switch (type) {
		case 0:
			{
			unsigned char *ucbuf = (unsigned char *)buf;
			for (i=0; i<len; i+=5) {
				for (j=i; j<len && j<(i+5); j++)
					fprintf (file, "%12u", ucbuf[j]);
				fprintf (file, "\n");
			}
			break;
			}
		case 1:
			{
			char *cbuf = (char *)buf;
			for (i=0; i<len; i+=5) {
				for (j=i; j<len && j<(i+5); j++)
					fprintf (file, "%12d", cbuf[j]);
				fprintf (file, "\n");
			}
			break;
			}
		case 2:
			{
			unsigned short *usbuf = (unsigned short *)buf;
			for (i=0; i<len; i+=5) {
				for (j=i; j<len && j<(i+5); j++)
					fprintf (file, "%12u", usbuf[j]);
				fprintf (file, "\n");
			}
			break;
			}
		case 3:
			{
			short *sbuf = (short *)buf;
			for (i=0; i<len; i+=5) {
				for (j=i; j<len && j<(i+5); j++)
					fprintf (file, "%12d", sbuf[j]);
				fprintf (file, "\n");
			}
			break;
			}
		case 4:
			{
			unsigned long *ulbuf = (unsigned long *)buf;
			for (i=0; i<len; i+=5) {
				for (j=i; j<len && j<(i+5); j++)
					fprintf (file, "%12u", ulbuf[j]);
				fprintf (file, "\n");
			}
			break;
			}
		case 5:
			{
			long *lbuf = (long *)buf;
			for (i=0; i<len; i+=5) {
				for (j=i; j<len && j<(i+5); j++)
					fprintf (file, "%12d", lbuf[j]);
				fprintf (file, "\n");
			}
			break;
			}
		case 6:
			{
			double *fbuf = (double *)buf;
			for (i=0; i<len; i+=5) {
				for (j=i; j<len && j<(i+5); j++)
					fprintf (file, "%12g", fbuf[j]);
				fprintf (file, "\n");
			}
			break;
			}
	}
}

void
printit2d (FILE *file, void *buf, int len, int type) {
	int i;

	switch (type) {
		case 0:
			{
			unsigned char *ucbuf = (unsigned char *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12u ", ucbuf[i]);
			break;
			}
		case 1:
			{
			char *cbuf = (char *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d ", cbuf[i]);
			break;
			}
		case 2:
			{
			unsigned short *usbuf = (unsigned short *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12u ", usbuf[i]);
			break;
			}
		case 3:
			{
			short *sbuf = (short *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d ", sbuf[i]);
			break;
			}
		case 4:
			{
			unsigned long *ulbuf = (unsigned long *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12u ", ulbuf[i]);
			break;
			}
		case 5:
			{
			long *lbuf = (long *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12d ", lbuf[i]);
			break;
			}
		case 6:
			{
			double *fbuf = (double *)buf;
			for (i=0; i<len; i++) fprintf (file, "%12g ", fbuf[i]);
			break;
			}
	}
	fprintf (file, "\n");
}

char errbuf[256];

int
main(int argc, char *argv[])
{

/*	//connect to the experiment defined by the EXPERIMENTDIR environment
		// variable
*/

	int             i;
	int             lim1, lim2, lim3, lim4, val;
	int            *counts;
	FILE           *file = stdout;
	char *caps = "capabilities";

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";
	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	prog = argv[0];
	
	if (argc < 3 || argc > 10) {
		usage ("incorrect number of arguments");
	}

	for (argv++, --argc ; argc > 0; argv+=2, argc-=2) {
		if (**argv != '-')  {
		usage ("incorrect argument format");
		} else {
			if (!strcmp("spectrum", ((*argv)+1))) {
				fromspec = argv[1];
			} else if (!strcmp("format", ((*argv)+1))) {
				format = argv[1];
			} else if (!strcmp("limit", ((*argv)+1))) {
				arglim1 = argv[1];
				if (argv[2] && *argv[2]!='-') {
					arglim2 = argv[2];
					argv++; argc--;
				}
				if (argv[2] && *argv[2]!='-') {
					arglim3 = argv[2];
					argv++; argc--;
				}
				if (argv[2] && *argv[2]!='-') {
					arglim4 = argv[2];
					argv++; argc--;
				}
			}
		}
	}

	if (!fromspec)
		crash("missing or invalid SPECTRUM argument");

	i = EGinquireSpectrum(fromspec, &dim, bases, ranges, layouts, types);
	if (i != 0)
	           readfailed();

	if (dim != 1 && dim != 2)
		notimplemented("spectrum not one or two dimensional");
	if (dim == 1) {
		lim1 = bases[0];
		if (arglim1) {
			i = sscanf(arglim1, "%i", &val);
			if (i != 1)
				usage1("invalid LIMIT argument");
			lim1 = val;
		}
	
		lim2 = bases[0]+ ranges[0] - 1;
		if (arglim2) {
			i = sscanf(arglim2, "%i", &val);
			if (i != 1)
				usage1("invalid LIMIT argument");
			lim2 = val;
		}
	
		if (lim1-bases[0] < 0 || lim1-bases[0] >= ranges[0])
			crash ("first limit lies ouside spectrum");
	
		if (lim2-bases[0] < 0 || lim2-bases[0] >= ranges[0])
			crash ("second limit lies ouside spectrum");

		bases[0] = MIN(lim1, lim2);
		ranges[0] = MAX(lim2, lim1) + 1 - bases[0];

		counts = (int *) malloc(ranges[0] * widths[types[0]]);

		i = EGread1dSpectrum(fromspec, bases[0], ranges[0], counts, types[0]);
		if (i != 0)
			readfailed();
		if (!format || *format=='\0' || !strcmp("x", format)) {
			printitx (file, counts, ranges[0], types[0]);
		} else if (!strcmp("xy", format)) {
			printitxy (file, counts, ranges[0], types[0]);
		} else if (!strcmp("5y", format)) {
			printit5y (file, counts, ranges[0], types[0]);
		} else {
			usage1 ("invalid FORMAT argument");
		}
		free(counts);
	} else if (dim == 2) {
		int c;

/*
		if (!arglim3) arglim3 = arglim1;
		if (!arglim4) arglim4 = arglim2;
*/

		if (arglim1) {
			i = sscanf(arglim1, "%i", &val);
			if (i != 1)
				usage2("invalid LIMIT argument");
			lim1 = val;
		} else 
			lim1 = bases[0];
	
		if (arglim2) {
			i = sscanf(arglim2, "%i", &val);
			if (i != 1)
				usage2("invalid LIMIT argument");
			lim2 = val;
		} else
			lim2 = bases[0] + ranges[0] - 1;

		if (arglim3) {
			i = sscanf(arglim3, "%i", &val);
			if (i != 1)
				usage2("invalid LIMIT argument");
			lim3 = val;
		} else 
			lim3 = bases[1];
	
		if (arglim4) {
			i = sscanf(arglim4, "%i", &val);
			if (i != 1)
				usage2("invalid LIMIT argument");
			lim4 = val;
		} else
			lim4 = bases[1] + ranges[1] - 1;
	
		if (lim1-bases[0] < 0 || lim1-bases[0] >= ranges[0])
			crash ("first limit lies ouside spectrum");
	
		if (lim2-bases[0] < 0 || lim2-bases[0] >= ranges[0])
			crash ("second limit lies ouside spectrum");

		if (lim3-bases[1] < 0 || lim3-bases[1] >= ranges[1])
			crash ("third limit lies ouside spectrum");
	
		if (lim4-bases[1] < 0 || lim4-bases[1] >= ranges[1])
			crash ("fourth limit lies ouside spectrum");

		bases[0] = MIN(lim1, lim2);
		ranges[0] = MAX(lim2, lim1) + 1 - bases[0];
		bases[1] = MIN(lim3, lim4);
		ranges[1] = MAX(lim4, lim3) + 1 - bases[1];

		if (!format || *format=='\0' || !strcmp("c", format)) {
			int *counts = (int *) malloc(ranges[1] * widths[types[0]]);
			for (c=bases[0]; c < (bases[0] + ranges[0]); c++) {
				i = EGread2dSpectrum(fromspec, c, 1, bases[1], ranges[1], counts, types[0]);
				if (i != 0)
					readfailed();
				printit2d (file, counts, ranges[1], types[0]);
			}
			free(counts);
		} else if (!strcmp("r", format)) {
			int *counts = (int *) malloc(ranges[0] * widths[types[0]]);
			for (c=bases[1]; c < (bases[1] + ranges[1]); c++) {
				i = EGread2dSpectrum(fromspec, bases[0], ranges[0], c, 1, counts, types[0]);
				if (i != 0)
					readfailed();
				printit2d (file, counts, ranges[0], types[0]);
			}
			free(counts);
		} else {
			usage2 ("invalid FORMAT argument");
		}
	}
	fprintf(stderr, "converted spectrum %s to ASCII\n", fromspec);
	return 0;
}
