#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>

#include "eglib.h"

#ifndef NULL
#define NULL 0
#endif

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
char           *fromspec;
char           *tospec;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;


void 
readfailed()
{
	char           *buf;
	buf = EGerrorMessage(EGerrno);
	printf("%s: %s\n", buf, fromspec);
	exit(1);
}

void 
writefailed()
{
	printf("Failed to write to output file\n");
	exit(1);
}

void 
notimplemented(char *buf)
{
	printf("%s: %s\n", "not implemented", buf);
	exit(1);
}

void 
EGcrash()
{
	char           *buf;
	buf = EGerrorMessage(EGerrno);
	printf("%s\n", buf);
	exit(1);
}

void 
crash(char *buf)
{
	printf("%s\n", buf);
	exit(1);
}


main(int argc, char *argv[])
{

/*	//connect to the experiment defined by the EXPERIMENTDIR environment
		// variable
*/

	int             i;
	int             n;
	char           buf[256];
	char   type[20], unit[20];
	float y,m,c;
	int            *counts;
	char           *format;
	FILE           *file;
	char *caps = "capabilities";

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	if (argc < 3)
		crash("Usage: spec-to-ascii fromspec tospec format");
	fromspec = argv[1];
	tospec = argv[2];
	format = argv[3];

	file = fopen (tospec, "w");
	if (file == NULL)
		crash ("failed to open output file\n");

	i = EGinquireSpectrum(fromspec, &dim, bases, ranges, layouts, types);
	if (i != 0)
	           readfailed();
	if (dim != 1 )
		notimplemented("spectrum not one dimensional");
	if (dim == 1)
	{
		counts = (int *) malloc(ranges[0] * widths[types[0]]);

		for (n = 1; n <= 16; n++) {
			*buf = '\0';
			i = EGreadInformation(fromspec, n, buf);
			if (i != 0)
				readfailed();
			if (*buf != '\0')
			{
				i = fprintf (file, "%s\n", buf);
				if (i < 0)
					writefailed();
			}
		}

		i = EGread1dSpectrum(fromspec, bases[0], ranges[0], counts, types[0]);
		if (i != 0)
			readfailed();

		i = EGreadCalibration(fromspec, 1, buf);
		if (i != 0)
			readfailed();
		i = sscanf (buf,"%s %s %d %f %f", type, unit, &n, &c, &m);
		if (i < 5)
		{
			for (n=0; n < ranges[0]; n++)
				fprintf (file, "%10d %10d\n", n, counts[n]);
		}
		else
		{
			for (n=0; n < ranges[0]; n++)
			{
				y = m * (n + bases[0]) + c;
				fprintf (file, "%10f %10d\n", y, counts[n]);
			}
		}


		free((char *) counts);
	}

	fclose (file);
	printf("copied spectrum %s to ASCII file %s\n", fromspec, tospec);
	return 0;
}
